from Products.ZPatterns.Agents import Agent, defaultConstructors
import urllib, sys
from zLOG import LOG, WARNING

class CatalogTrigger(Agent):
    """ Catalog/Recatalog/Uncatalog objects on add/change/delete 
    
    This trigger can be used as a replacement for CatalogAwareness.
    """
    
    meta_type = "Cataloging Trigger"
    
    _properties=(
        {'id':'default_catalog', 'type': 'string', 'mode': 'w'},
        {'id':'handle_change','type': 'boolean', 'mode': 'w'},
    )
    
    propertyLabels = {
        'default_catalog' : "ZCatalog's id:",
        'handle_change' : 'Recatalog when objects change:',
    }
    
    def __init__(self, id, title=''):
        Agent.__init__(self, id, title)
        self.__dict__['default_catalog'] = "Catalog"
        self.__dict__['handle_change'] = 1
        
    # Cataloging methods - should be updated from the latest version of
    # CatalogAware when we move to to 2.2
    def catalog_object(self, object):
        """A common method to allow Findables to index themselves."""
        if hasattr(self, self.default_catalog):
            getattr(self, self.default_catalog).catalog_object(object, url(object))

    def uncatalog_object(self, object):
        """A common method to allow Findables to unindex themselves."""
        try:
            if hasattr(self, self.default_catalog):
                getattr(self, self.default_catalog).uncatalog_object(url(object))
        except:
            pass

    def recatalog_object(self, object):
        """ Suprisingly useful """       
        self.uncatalog_object(object)
        self.catalog_object(object)
    
    # Event handlers, overriding Agent's defaults
    def _getMementoFor(self,client):
        return client.__dict__
        
    def _objectChanged(self,client,memento):
        if self.handle_change:
            self.recatalog_object(client)

    def _objectDeleted(self,client,memento):
        self.uncatalog_object(client)

    def _objectAdded(self,client):
        try: self.catalog_object(client)
        except:
            LOG('ZPAddons',WARNING,('Catalogging failed for %s' % client),
                '', sys.exc_info(), 1
            )
            
def url(object, ftype=urllib.splittype, fhost=urllib.splithost):
    """Return a SCRIPT_NAME-based url for an object."""
    if hasattr(object, 'DestinationURL') and \
       callable(object.DestinationURL):
        url='%s/%s' % (object.DestinationURL(), object.id)
    else: url=object.absolute_url()
    type, uri=ftype(url)
    host, uri=fhost(uri)
    script_name=object.REQUEST['SCRIPT_NAME']
    __traceback_info__=(`uri`, `script_name`)
    if script_name:
        uri=filter(None, string.split(uri, script_name))[0]
    uri=uri or '/'
    return urllib.unquote(uri)
    
def initialize(context):

    context.registerPlugInClass(
        CatalogTrigger,
        permission = 'Add Generic Triggers',
        constructors = defaultConstructors(CatalogTrigger,globals()),
        icon = 'www/catalogtrigger.gif'
    )    
    
